package de.moritzruth.dracula_musical.device

import de.moritzruth.theaterdsl.device.DeviceGroup
import de.moritzruth.theaterdsl.dmx.DmxAddress
import kotlinx.collections.immutable.persistentSetOf

object FrontLights {
    val left = DeviceGroup(SimpleDimmer(DmxAddress(1u)), SimpleDimmer(DmxAddress(2u)))
    val center = DeviceGroup(SimpleDimmer(DmxAddress(4u)), SimpleDimmer(DmxAddress(5u)))
    val right = DeviceGroup(SimpleDimmer(DmxAddress(7u)), SimpleDimmer(DmxAddress(8u)))
    val all = left + center + right
}

object Spots {
    val left = SimpleDimmer(DmxAddress(10u))
    val right = SimpleDimmer(DmxAddress(11u))
}

object Tops {
    val left = FuturelightDmh160(DmxAddress(37u), false)
    val right = FuturelightDmh160(DmxAddress(53u), true)
    val both = DeviceGroup(left, right)
}

object Washs {
    val left = CoemarProWash(DmxAddress(101u), false)
    val right = CoemarProWash(DmxAddress(85u), true)
    val both = DeviceGroup(left, right)
}

object BlinderBars {
    val inner = DeviceGroup(StairvilleSplb(DmxAddress(155u)), StairvilleSplb(DmxAddress(178u)))
    val outer = DeviceGroup(StairvilleSplb(DmxAddress(201u)), StairvilleSplb(DmxAddress(224u)))
    val all = inner + outer
}

val backlightBar = StairvilleTlb(DmxAddress(130u))
val sidelight = StairvilleClb4(DmxAddress(140u))

val fogMachine = AdjFogFury3000(DmxAddress(117u))

val devices = persistentSetOf(
    *FrontLights.all.toTypedArray(),
    Spots.left,
    Spots.right,
    *Tops.both.toTypedArray(),
    *Washs.both.toTypedArray(),
    *BlinderBars.all.toTypedArray(),
    backlightBar,
    sidelight,
    fogMachine
)