package de.moritzruth.dracula_musical.device

import de.moritzruth.theaterdsl.device.*
import de.moritzruth.theaterdsl.dmx.DmxAddress
import de.moritzruth.theaterdsl.dmx.DmxDataWriter
import de.moritzruth.theaterdsl.dmx.DmxValue
import de.moritzruth.theaterdsl.value.Color
import de.moritzruth.theaterdsl.value.percent
import kotlinx.collections.immutable.ImmutableSet
import kotlinx.collections.immutable.persistentSetOf
import kotlin.math.roundToInt

class StairvilleSplb(override val firstChannel: DmxAddress) : Device {
    override val numberOfChannels: UInt = 11u

    val brightness = PercentageDV()
    val strobeFrequency = PercentageDV()

    val color = ColorDV(Color.WHITE.copy(brightness = 0.percent))
    val white = PercentageDV()

    val preset = ConcreteDV<Preset?>(null)
    val presetSpeed = PercentageDV()

    enum class Preset(channelValue: UByte) {
        STUB(16u);

        val channelValue: DmxValue = DmxValue(channelValue)
    }

    override val dvs: ImmutableSet<DynamicValue<*>> = persistentSetOf(
        brightness,
        strobeFrequency,
        color,
        white,
        preset,
        presetSpeed
    )

    override fun writeDmxData(writer: DmxDataWriter, isLightBehindCurtainOn: Boolean) {
        writer.writePercentage(brightness.getCurrentValue())

        val strobeFrequency = strobeFrequency.getCurrentValue()
        if (strobeFrequency == 0.percent) writer.writeRaw(DmxValue(0u))
        else writer.writeRaw(DmxValue(strobeFrequency.ofRange(11.0..255.0).roundToInt().toUByte()))

        // mode
        val preset = preset.getCurrentValue()
        if (preset == null) writer.writeRaw(DmxValue(0u)) // mode = RGBW
        else writer.writeRaw(DmxValue(128u)) // mode = preset

        writer.writeRaw(DmxValue(0u)) // color preset (not used)

        if (preset == null) writer.writeRaw(DmxValue(0u))
        else writer.writeRaw(preset.channelValue)

        writer.writeRaw(DmxValue(0u)) // sound mode preset

        writer.writePercentage(presetSpeed.getCurrentValue())

        val (red, green, blue) = color.getCurrentValue().getRGB()
        writer.writePercentage(red)
        writer.writePercentage(green)
        writer.writePercentage(blue)
        writer.writePercentage(white.getCurrentValue())
    }
}