package de.moritzruth.dracula_musical.device

import de.moritzruth.theaterdsl.device.ColorDV
import de.moritzruth.theaterdsl.device.Device
import de.moritzruth.theaterdsl.device.PercentageDV
import de.moritzruth.theaterdsl.dmx.DmxAddress
import de.moritzruth.theaterdsl.dmx.DmxDataWriter
import de.moritzruth.theaterdsl.value.Percentage
import kotlinx.collections.immutable.persistentSetOf
import kotlin.math.pow

class StairvilleTlb(override val firstChannel: DmxAddress) : Device {
    override val numberOfChannels: UInt = 5u

    override fun writeDmxData(writer: DmxDataWriter, isLightBehindCurtainOn: Boolean) {
        val (red, green, blue) = color.getCurrentValue().getRGB()
        writer.writePercentage(red)
        writer.writePercentage(green)
        writer.writePercentage(blue)

        val adjustedBrightness = brightness.getCurrentValue().value.pow(2.5)
        writer.writePercentage(Percentage(adjustedBrightness))
        writer.writePercentage(strobeSpeed.getCurrentValue())
    }

    val color = ColorDV()
    val brightness = PercentageDV()
    val strobeSpeed = PercentageDV()

    override val dvs = persistentSetOf(
        color,
        brightness,
        strobeSpeed
    )
}