package de.moritzruth.dracula_musical.song

import de.moritzruth.dracula_musical.device.*
import de.moritzruth.theaterdsl.show.SceneBuilderContext
import de.moritzruth.theaterdsl.show.StepCue
import de.moritzruth.theaterdsl.value.Color
import de.moritzruth.theaterdsl.value.degrees
import de.moritzruth.theaterdsl.value.percent
import kotlin.time.Duration.Companion.minutes
import kotlin.time.Duration.Companion.seconds

fun SceneBuilderContext.songDuettDraculaMina() {
    step(StepCue.MusicStart("Duett: Dracula & Mina", 4.minutes + 30.seconds)) {
        rightSpotTarget = "Mina"
        rightSpotTarget = "Dracula"

        onRun {
            FrontLights.all { it.brightness.fade(25.percent, 30.seconds) }
            Spots.right.brightness.fade(100.percent, 30.seconds)
            Spots.left.brightness.fade(100.percent, 30.seconds)

            backlightBar.brightness.off(20.seconds)

            sidelight.brightness.fade(50.percent, 50.seconds)
            sidelight.color1.fadeRandomAround(25.degrees, 30.degrees, 10.seconds)
            sidelight.color2.fadeRandomAround(230.degrees, 30.degrees, 10.seconds)
            sidelight.color3.fadeRandomAround(25.degrees, 30.degrees, 10.seconds)
            sidelight.color4.fadeRandomAround(230.degrees, 30.degrees, 10.seconds)
        }
    }

    lightStep(StepCue.Custom("Musik: Strophe")) {
        FrontLights.all { it.brightness.off(15.seconds) }
    }

    lightStep(StepCue.Text("Mina", "Ich kenne dich!", "Anfang")) {
        BlinderBars.all {
            it.preset.static(StairvilleSplb.Preset.STUB)  // flowy, dreamy
            it.presetSpeed.static(10.percent)
            it.brightness.fade(25.percent, 2.seconds)
        }

        Washs.both {
            it.pointAtAudience()
            it.colorWheelMode.static(CoemarProWash.ColorWheelMode.DarkBlue)
        }

        Tops.both {
            it.startRoomMovement(2.5)
            it.colorWheelMode.static(FuturelightDmh160.ColorWheelMode.Rotate(10.percent))
            it.prismMode.static(FuturelightDmh160.PrismMode.FACETS_8)
            it.prismRotationSpeed.static(5.percent)
        }
    }

    lightStep(StepCue.Text("Dracula", "…mit aller Kraft!", "letzte Silbe")) {
        Washs.both {
            it.pointAtCeiling(9.seconds)
            it.brightness.fade(100.percent, 3.seconds)
        }
    }

    lightStep(StepCue.Text("Mina", "Komm zu mir!", "Anfang")) {
        Tops.both { it.brightness.fade(100.percent, 3.seconds) }
    }

    lightStep(StepCue.Text("Dracula", "Komm zu mir!", "Anfang")) {
        Washs.both { it.brightness.off(4.seconds) }
    }

    lightStep(StepCue.Text("Dracula & Mina", "Komm!", "Anfang")) {
        Tops.both { it.brightness.off(4.seconds) }
        Washs.both {
            it.pointAtStageCenter()
            it.colorWheelMode.static(CoemarProWash.ColorWheelMode.Red)
            it.beamAngle.static(0.percent)
        }

        Spots.right.brightness.off(6.seconds)
        sidelight.brightness.off(6.seconds)

        backlightBar.color.static(Color.RED)
        backlightBar.brightness.fade(100.percent, 6.seconds)
    }

    lightStep(StepCue.MusicEnd) {
        Spots.left.brightness.off(3.seconds)

        backlightBar.brightness.pulse(1.8.seconds, 0.15.seconds, 0.2.seconds, startDelay = 0.seconds)
        sidelight.brightness.pulse(1.8.seconds, 0.15.seconds, 0.3.seconds, startDelay = 0.4.seconds)
        Washs.both { it.brightness.pulse(1.8.seconds, 0.15.seconds, 0.3.seconds, startDelay = 0.4.seconds) }
    }
}