package de.moritzruth.dracula_musical.song

import de.moritzruth.dracula_musical.device.*
import de.moritzruth.theaterdsl.show.SceneBuilderContext
import de.moritzruth.theaterdsl.show.StepCue
import de.moritzruth.theaterdsl.value.Color
import de.moritzruth.theaterdsl.value.degrees
import de.moritzruth.theaterdsl.value.percent
import kotlin.time.Duration.Companion.minutes
import kotlin.time.Duration.Companion.seconds

fun SceneBuilderContext.songEsIstAngerichtet() {
    step(StepCue.MusicStart("Es ist angerichtet", 2.minutes + 30.seconds)) {
        actors {
            // TODO: Expand
            +"Vampire"
        }

        onRun {
            backlightBar.color.static(Color(hue = 0.degrees, saturation = 80.percent))
            backlightBar.brightness.pulse(1.8.seconds, 0.15.seconds, 0.2.seconds, startDelay = 0.seconds, peak = 20.percent)

            Washs.both {
                it.pointAtStageCenter()
                it.colorWheelMode.static(CoemarProWash.ColorWheelMode.Red)
                it.brightness.pulse(1.8.seconds, 0.15.seconds, 0.3.seconds, startDelay = 0.4.seconds, peak = 30.percent)
            }
        }
    }

    lightStep(StepCue.Custom("Musik: Einsetzen der Drums")) {
        backlightBar.brightness.pulse(1.6.seconds, 0.15.seconds, 0.2.seconds, startDelay = 0.seconds, peak = 20.percent)
        Washs.both { it.brightness.pulse(1.6.seconds, 0.1.seconds, 0.3.seconds, startDelay = 0.4.seconds, peak = 100.percent) }

        BlinderBars.all {
            it.preset.static(StairvilleSplb.Preset.STUB)  // red, chase, back-and-forth
            it.presetSpeed.static(50.percent)
            it.brightness.fade(10.percent, 10.seconds)
        }
    }


    lightStep(StepCue.Text("Vampire", "Wir wollen bloß spielen!", "Anfang")) {
        Washs.both {
            it.brightness.off(0.3.seconds)
            it.pointAtAudience()
        }
    }

    repeat(2) {
        lightStep(StepCue.Text("Vampire", "Aaah")) {
            sidelight.brightness.fade(80.percent, 3.seconds)
            backlightBar.brightness.pulse(0.5.seconds, 0.1.seconds, 0.2.seconds, startDelay = 0.seconds, peak = 20.percent)
        }

        lightStep(StepCue.Text("Vampire", "…angerichtet!", "erste Silbe")) {
            backlightBar.brightness.pulseOnce(0.1.seconds, 2.seconds, end = 20.percent)
        }

        lightStep(StepCue.Text("Vampire", "…angerichtet!", "zweite Silbe")) {
            sidelight.brightness.pulseOnce(0.1.seconds, 1.seconds, end = 40.percent)
            Washs.both { it.brightness.pulseOnce(0.1.seconds, 0.4.seconds) }
        }

        lightStep(StepCue.Text("Vampire", "…angerichtet!", "dritte Silbe")) {
            BlinderBars.outer { it.brightness.pulseOnce(0.1.seconds, 0.4.seconds, peak = 50.percent, end = 10.percent) }
        }

        lightStep(StepCue.Text("Vampire", "…angerichtet!", "vierte Silbe")) {
            BlinderBars.inner { it.brightness.pulseOnce(0.1.seconds, 0.4.seconds, peak = 50.percent, end = 10.percent) }
        }
    }

    lightStep(StepCue.Text("Vampire", "Aaah")) {
        sidelight.brightness.fade(80.percent, 3.seconds)
        backlightBar.brightness.pulse(1.seconds, 0.1.seconds, 0.2.seconds, startDelay = 0.seconds, peak = 40.percent)
    }

    lightStep(StepCue.Text("Vampire", "Aaah")) {
        // Empty to prevent confusion with the next step
    }

    repeat(2) {
        lightStep(StepCue.Text("Vampire", "Aaah")) {
            sidelight.brightness.fade(80.percent, 3.seconds)
            backlightBar.brightness.pulse(0.5.seconds, 0.1.seconds, 0.2.seconds, startDelay = 0.seconds, peak = 20.percent)
        }

        lightStep(StepCue.Text("Vampire", "…angerichtet!", "erste Silbe")) {
            backlightBar.brightness.pulseOnce(0.1.seconds, 2.seconds, end = 20.percent)
        }

        lightStep(StepCue.Text("Vampire", "…angerichtet!", "zweite Silbe")) {
            sidelight.brightness.pulseOnce(0.1.seconds, 1.seconds, end = 40.percent)
            Washs.both { it.brightness.pulseOnce(0.1.seconds, 0.4.seconds) }
        }

        lightStep(StepCue.Text("Vampire", "…angerichtet!", "dritte Silbe")) {
            BlinderBars.outer { it.brightness.pulseOnce(0.1.seconds, 0.4.seconds, peak = 50.percent, end = 10.percent) }
        }

        lightStep(StepCue.Text("Vampire", "…angerichtet!", "vierte Silbe")) {
            BlinderBars.inner { it.brightness.pulseOnce(0.1.seconds, 0.4.seconds, peak = 50.percent, end = 10.percent) }
        }
    }

    lightStep(StepCue.Text("Vampire", "Aaah")) {
        sidelight.brightness.pulseOnce(2.seconds, 1.seconds, peak = 80.percent, end = 40.percent)
        backlightBar.brightness.pulse(1.seconds, 0.1.seconds, 0.2.seconds, startDelay = 0.seconds, peak = 40.percent)

        Washs.both {
            it.pointAtAudience()
            it.colorWheelMode.static(CoemarProWash.ColorWheelMode.WarmWhite)
            it.beamAngle.static(100.percent)
        }
    }

    lightStep(StepCue.Text("Vampire", "Aaah")) {
        sidelight.brightness.pulseOnce(2.seconds, 1.seconds, peak = 80.percent, end = 40.percent)

        BlinderBars.all { it.brightness.pulseOnce(2.seconds, 1.seconds, peak = 50.percent, end = 30.percent) }
        Washs.both {
            it.pointAtCeiling(3.seconds)
            it.brightness.pulseOnce(0.5.seconds, 3.seconds)
        }
    }

    lightStep(StepCue.Text("Vampire", "Aaah")) {
        sidelight.brightness.off(7.seconds)
        backlightBar.strobeSpeed.fade(start = 60.percent, end = 90.percent, duration = 7.seconds)
        backlightBar.brightness.fade(start = 10.percent, end = 100.percent, duration = 7.seconds)

        BlinderBars.all { it.brightness.off(5.seconds) }
    }

    lightStep(StepCue.Text("Vampire", "Jetzt…")) {
        backlightBar.brightness.static(100.percent)
        backlightBar.strobeSpeed.static(0.percent)
    }

    lightStep(StepCue.Text("Vampire", "…ist…")) {
        sidelight.brightness.static(100.percent)
    }

    lightStep(StepCue.Text("Vampire", "…Essenszeit!", "erste Silbe")) {
        Washs.both { it.brightness.static(100.percent) }
    }

    lightStep(StepCue.Text("Vampire", "…Essenszeit!", "zweite Silbe")) {
        BlinderBars.outer {
            it.preset.static(null)
            it.color.static(Color(hue = 0.degrees))
            it.brightness.static(100.percent)
        }
    }

    lightStep(StepCue.Text("Vampire", "…Essenszeit!", "dritte Silbe")) {
        BlinderBars.inner {
            it.preset.static(null)
            it.color.static(Color(hue = 0.degrees))
            it.brightness.static(100.percent)
        }
    }

    lightStep(StepCue.Custom("Musik: Nächster Beat")) {
        backlightBar.color.fade(Color(hue = 0.degrees, saturation = 30.percent), 1.seconds)
        backlightBar.brightness.fade(30.percent, 1.seconds)

        sidelight.brightness.off(1.seconds)
        Washs.both { it.brightness.off(1.seconds) }
        BlinderBars.all { it.brightness.off(1.seconds) }
    }

    lightStep(StepCue.MusicEnd) {
        sidelight.brightness.fade(40.percent, 10.seconds)

        BlinderBars.all { it.color.reset() }
    }
}