package de.moritzruth.dracula_musical.song

import de.moritzruth.dracula_musical.device.*
import de.moritzruth.theaterdsl.show.SceneBuilderContext
import de.moritzruth.theaterdsl.show.StepCue
import de.moritzruth.theaterdsl.value.Color
import de.moritzruth.theaterdsl.value.degrees
import de.moritzruth.theaterdsl.value.percent
import kotlin.time.Duration.Companion.minutes
import kotlin.time.Duration.Companion.seconds

fun SceneBuilderContext.songLucy() {
    lightStep(StepCue.MusicStart("Lucy", 2.minutes + 45.seconds)) {
        sidelight.brightness.fade(30.percent, 15.seconds)
        backlightBar.brightness.fade(40.percent, 15.seconds)
        backlightBar.color.static(Color(hue = 315.degrees))

        Washs.both {
            it.pointAtCeiling()
            it.colorWheelMode.static(CoemarProWash.ColorWheelMode.White)
            it.magenta.static(100.percent)
            it.beamAngle.static(100.percent)
        }

        Tops.both {
            it.startRoomMovement(4.0)
            it.colorWheelMode.static(FuturelightDmh160.ColorWheelMode.LightPink)
            it.prismMode.static(FuturelightDmh160.PrismMode.OPEN)
        }
    }

    // TODO: Do more during the first part

    lightStep(StepCue.Custom("Musik: Refrain")) {
        BlinderBars.all {
            it.brightness.fade(25.percent, 2.seconds)
            it.preset.static(StairvilleSplb.Preset.STUB)  // red, flowy
        }

        Washs.left.brightness.sine(6.seconds, start = 0.percent)
        Washs.right.brightness.sine(6.seconds, start = 100.percent)
    }

    lightStep(StepCue.Custom("Musik: Strophe")) {
        Tops.both { it.brightness.sine(8.seconds, 20.percent, 100.percent) }
    }

    lightStep(StepCue.Custom("Musik: Bridge")) {
        Tops.both { it.brightness.sine(8.seconds, 20.percent, 100.percent) }
        BlinderBars.all {
            it.brightness.fade(40.percent, 10.seconds)
            it.preset.static(StairvilleSplb.Preset.STUB)  // different, red, flowy
        }
    }

    lightStep(StepCue.Custom("Musik: Refrain")) {
        BlinderBars.all { it.brightness.fade(25.percent, 3.seconds) }
        // TODO
    }

    lightStep(StepCue.Text("Lucy", "oh Nick", "letzte Silbe")) {
        Tops.both { it.brightness.pulseOnce(0.1.seconds, 1.seconds) }
    }

    lightStep(StepCue.Custom("Musik: Interludium")) {
        // TODO
    }

    lightStep(StepCue.Custom("Musik: Refrain")) {
        // TODO
    }

    lightStep(StepCue.Custom("Musik: Schlussschlag")) {
        // TODO
        backlightBar.brightness.off(2.seconds)
        Washs.both { it.brightness.pulseOnce(0.1.seconds, 2.seconds) }
    }

    step(StepCue.MusicEnd) {
        Tops.both { it.pointAtStageMiddleSingleCenterMarking() }
        Washs.both { it.magenta.reset() }
        BlinderBars.all { it.brightness.off(3.seconds) }
        backlightBar.brightness.off(3.seconds)
    }
}