package de.moritzruth.dracula_musical.song

import de.moritzruth.dracula_musical.device.*
import de.moritzruth.theaterdsl.show.CurtainState
import de.moritzruth.theaterdsl.show.SceneBuilderContext
import de.moritzruth.theaterdsl.show.StepCue
import de.moritzruth.theaterdsl.value.Color
import de.moritzruth.theaterdsl.value.degrees
import de.moritzruth.theaterdsl.value.percent
import kotlin.time.Duration.Companion.minutes
import kotlin.time.Duration.Companion.seconds

fun SceneBuilderContext.songMittsommernacht() {
    step(StepCue.MusicStart("Mittsommernacht", 4.minutes + 30.seconds)) {
        actors {
            // TODO: Expand
            +"Einwohner von Huntington / von links & durch Mitte"
        }

        curtainState = CurtainState.OPEN

        onRun {
            FrontLights.all {
                it.brightness.fade(100.percent, 6.seconds)
            }

            Tops.both {
                it.startRoomMovement(3.0)
                it.brightness.fade(100.percent, 2.seconds)
                it.colorWheelMode.static(FuturelightDmh160.ColorWheelMode.Rotate(10.percent))
                it.prismMode.static(FuturelightDmh160.PrismMode.FACETS_8)
                it.prismRotationSpeed.static(20.percent)
            }

            backlightBar.brightness.fade(75.percent, 2.seconds)
            backlightBar.color.cycle(20.seconds)

            Washs.both {
                it.brightness.pulseOnce(2.seconds, 5.seconds, 100.percent)
                it.beamAngle.fade(0.percent, 2.seconds)
                it.pointAtCeiling(6.seconds)
            }

            BlinderBars.all {
                it.white.static(100.percent)
                it.brightness.pulseOnce(3.seconds, 5.seconds, 75.percent)
            }
        }
    }

    lightStep(StepCue.Custom("Musik: Drums-Pattern wechselt")) {
        BlinderBars.all {
            it.brightness.fade(25.percent, 1.seconds)
            it.white.static(0.percent)
            it.presetSpeed.static(20.percent)
            it.preset.static(StairvilleSplb.Preset.STUB)  // flowy, warm
        }
    }

    lightStep(StepCue.Custom("Musik: Strophe")) {
        Tops.both {
            it.brightness.off(2.seconds)
        }

        BlinderBars.all {
            it.presetSpeed.static(50.percent)
            it.preset.static(StairvilleSplb.Preset.STUB)  // jumpy, warm
        }
    }

    // TODO: Fix speaker
    step(StepCue.Text("Girl 1", "Hilfe, das Kuschel-Monster!", "Anfang")) {
        rightSpotTarget = "verängstigte Mädchen (Mitte der Bühne)"

        onRun {
            Spots.right.brightness.static(100.percent)

            FrontLights.all {
                it.brightness.static(25.percent)
            }

            backlightBar.color.fadeRandomAround(0.degrees, 5.degrees, 2.seconds)
            backlightBar.brightness.fade(50.percent, 1.seconds)

            BlinderBars.all {
                it.brightness.off(1.seconds)
                it.presetSpeed.static(5.percent)
                it.preset.static(StairvilleSplb.Preset.STUB)  // back-and-forth
            }
        }
    }

    // TODO: Fix speaker
    step(StepCue.Text("Girls", "Bitte tut uns nichts!", "Ende")) {
        rightSpotTarget = null

        onRun {
            Spots.right.brightness.off(2.seconds)

            FrontLights.all {
                it.brightness.fade(100.percent, 8.seconds)
            }

            backlightBar.brightness.off(6.seconds)
        }
    }

    lightStep(StepCue.Custom("Musik: Refrain")) {
        BlinderBars.all {
            it.brightness.fade(25.percent, 1.seconds)
            it.presetSpeed.static(50.percent)
            it.preset.static(StairvilleSplb.Preset.STUB)  // jumpy, warm
        }

        backlightBar.brightness.fade(75.percent, 2.seconds)
        backlightBar.color.cycle(12.seconds)

        Washs.both {
            it.pointAtStageCenter()
            it.beamAngle.static(100.percent)
        }
    }

    lightStep(StepCue.Text("Alle", "Heute ist die Mittsommer…", "letzte Silbe")) {
        BlinderBars.all {
            it.brightness.fade(start = 40.percent, end = 0.percent, duration = 5.seconds)
            it.white.static(100.percent)
            it.preset.static(null)
        }

        backlightBar.color.static(Color(hue = 40.degrees, saturation = 50.percent))

        Washs.both { it.brightness.static(100.percent) }
    }

    step(StepCue.Custom("Musik: 1. Hit")) {
        actors {
            +"Dracula / durch Mitte"
            +"Eloïse / durch Mitte"
            +"Françoise / durch Mitte"
            +"Bernadette / durch Mitte"
        }

        onRun {
            FrontLights.all { it.brightness.off() }
        }
    }

    lightStep(StepCue.Custom("Musik: 2. Hit")) {
        backlightBar.brightness.off(0.5.seconds)
    }

    lightStep(StepCue.Custom("Musik: 3. Hit")) {
        Washs.both { it.brightness.off(0.5.seconds) }
    }

    lightStep(StepCue.Custom("Musik: 4. Hit")) {
        BlinderBars.all { it.brightness.pulseOnce(0.1.seconds, 0.3.seconds, 100.percent) }

        Washs.both {
            it.colorWheelMode.static(CoemarProWash.ColorWheelMode.Red)
        }
    }

    lightStep(StepCue.Custom("Musik: 5. Hit")) {
        BlinderBars.all {
            it.white.off()
            it.color.static(Color(hue = 345.degrees))
            it.brightness.pulseOnce(0.1.seconds, 2.seconds, 100.percent)
        }

        backlightBar.color.static(Color(hue = 240.degrees))
        backlightBar.brightness.fade(75.percent, 3.seconds)

        Washs.both { it.brightness.fade(100.percent, 3.seconds) }
    }

    step(StepCue.Text("Vampire", "Süße Damen")) {
        leftSpotTarget = "Vampire (sprunghaft hin und her)"
        rightSpotTarget = "Vampire (sprunghaft hin und her)"

        onRun {
            Spots.left.brightness.static(50.percent)
            Spots.right.brightness.static(50.percent)

            BlinderBars.all {
                it.color.reset()
                it.brightness.fade(25.percent, 1.seconds)
                it.preset.static(StairvilleSplb.Preset.STUB)  // chase, red
                it.presetSpeed.static(80.percent)
            }
        }
    }

    step(StepCue.Text("Vampire", "Wir warten auf euch", "Anfang")) {
        rightSpotTarget = null

        onRun {
            Spots.right.brightness.off(3.seconds)
            Washs.both { it.brightness.off(2.seconds) }

            BlinderBars.all {
                it.presetSpeed.fade(5.percent, 2.seconds)
                it.brightness.fade(60.percent, 10.seconds)
            }
        }
    }

    step(StepCue.Text("Vampire", "Schloss Dracula", "Anfang")) {
        onRun {
            Spots.left.brightness.fade(100.percent, 3.seconds)
            backlightBar.brightness.fade(100.percent, 3.seconds)

            Washs.both {
                it.pointAtAudience()
                it.colorWheelMode.static(CoemarProWash.ColorWheelMode.WarmWhite)
            }
        }
    }

    step(StepCue.Text("Vampire", "Schloss Dracula", "Ende")) {
        actors {
            -"Dracula"
            -"Eloïse"
            -"Françoise"
            -"Bernadette"
        }

        leftSpotTarget = null

        onRun {
            Spots.left.brightness.off()
            backlightBar.brightness.off(1.seconds)
            BlinderBars.all { it.brightness.off(1.seconds) }
        }
    }

    lightStep(StepCue.Custom("Die Bewohner entfreezen")) {
        FrontLights.all { it.brightness.fade(100.percent, 1.seconds) }
        Tops.both { it.brightness.fade(100.percent, 2.seconds) }

        backlightBar.brightness.fade(75.percent, 2.seconds)
        backlightBar.color.cycle(12.seconds)

        Washs.both {
            it.brightness.pulseOnce(2.seconds, 5.seconds, 100.percent)
            it.beamAngle.fade(0.percent, 2.seconds)
            it.pointAtCeiling(6.seconds)
        }

        BlinderBars.all {
            it.brightness.fade(30.percent, 3.seconds)
            it.presetSpeed.static(20.percent)
            it.preset.static(StairvilleSplb.Preset.STUB)  // flowy, warm
        }
    }

    lightStep(StepCue.Text("Alle", "In der Mittsommernacht!", "Anfang")) {
        Tops.both { it.brightness.off(3.seconds) }
        backlightBar.brightness.off(3.seconds)
        BlinderBars.all { it.brightness.off(3.seconds) }
    }

    lightStep(StepCue.Custom("Musik: 1. Hit")) {
        backlightBar.color.static(Color(hue = 40.degrees))
        backlightBar.brightness.fade(100.percent, 0.2.seconds)
        Washs.both { it.brightness.fade(100.percent, 0.2.seconds) }
    }

    lightStep(StepCue.Custom("Musik: 2. Hit")) {
        BlinderBars.all {
            it.white.static(100.percent)
            it.preset.static(null)
            it.brightness.pulseOnce(0.2.seconds, 2.seconds, 100.percent)
        }

        backlightBar.brightness.off(3.seconds)
        Washs.both { it.brightness.off(3.seconds) }
    }

    step(StepCue.MusicEnd) {
        actors {
            -"Einwohner von Huntington"
            +"Mina / durch Mitte, mit Koffer & Taschentuch"
            +"Lucy / durch Mitte"
            +"Jonathan / von links"
        }

        onRun {
            FrontLights.all { it.brightness.fade(50.percent, 3.seconds) }
        }
    }
}